//
//  STFramedObject.h
//  Kahuna
//
//  Created by Davyd Geyl on 28/08/09.
//  Copyright 2009 Sportstec Ltd. All rights reserved.
//


#import <SCNFoundation/STProtoObject.h>
#import <SCNFoundation/STSorting.h>

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

extern NSInteger const kSTFramedObjectVersionCODAv1;
extern NSInteger const kSTFramedObjectVersionKahuna; // used in CODA/iCODA v2.x, GameBreaker 1.x

extern NSInteger const kSTFramedObjectVersionCODAv3; // will be used in CODA/iCODA v3.x, including new STFormContainer

extern NSInteger const kSTFramedObjectVersionProject77Prototype;

    #define kSTFramedObjectVersion kSTFramedObjectVersionProject77Prototype

extern NSInteger const kSTFramedObjectInitialZedOrder;

@interface STFramedObject : STProtoObject <NSCoding, STHeldObjectInterface, STFramedObjectInterface, STAnimatedVisibleInterface, STSortableItemInterface>
{
    CGRect _frame;
}

+ (CGSize)minimalSize;
+ (CGSize)maximumFieldSize;

- (instancetype)initWithFrame:(CGRect)frame;

@property (nonatomic, assign) id<STFramedContainerInterface> container;

@property (nonatomic, assign, getter=isAnimatedVisible) BOOL animatedVisible;
@property (nonatomic, assign) CGFloat zedOrder;
@property (nonatomic, assign) NSUInteger storedHash;
@property (nonatomic, assign, getter=isLocked) BOOL locked;
@property (nonatomic, readonly) NSInteger objectVersion;

@property (nonatomic, assign) CGRect frame;

@property (nonatomic, assign) CGPoint position;

@property (nonatomic, readonly) CGRect frameInContainer; // This is a temp solution because popup tags are located on sub-layer of masterbutton, but container needs to know its frame. Will be refactored when we decide to let code button be container.

// TODO: We actually return container for the sortmaster value now - this may break some (disabled) drawing code.
@property (nonatomic, assign) id<STSortMasterInterface> sortMaster;


- (CGRect)validFrameWithFrame:(CGRect)frame;
- (void)userRemoveFromSuperContainer;
- (void)removeFromSuperContainer;

- (void)setUpObjectWithSelf:(STFramedObject*)object;

- (CGRect)frameInContainer:(id<STFramedContainerInterface>)container;
- (CGRect)frameInRootContainer;
- (CGPoint)positionInContainer:(id<STFramedContainerInterface>)container;

//for subclasses
- (void)validateBounds;
- (void)validatePosition;
- (void)setBoundsAndNotify:(CGRect)bounds;

@end
