//
//  STMacOSProtocols.h
//  Kahuna
//
//  Created by Davyd Geyl on 6/10/10.
//  Copyright 2010 Sportstec Ltd. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AppKit/AppKit.h>

@class STSelectionController;

extern NSString* const STDocumentDidBecomeActiveNotification;
extern NSString* const STDocumentWillCloseNotification;

extern NSString* const STTimelineDraggingPasteboardType;
extern NSString* const STStackBoardDraggingPasteboardType;

@protocol STSelectionControllerHolderInterface

- (STSelectionController*)selectionController;

@end

#pragma mark - Generic Color

Protocol* STGenericColorInterfaceRef(void);

@protocol STGenericColorInterface
- (NSColor*)genericColor;
- (void)setGenericColor:(NSColor*)color;
@end

#pragma mark - Generic Font

Protocol* STGenericFontInterfaceRef(void);

@protocol STGenericFontInterface
- (NSFont*)genericFont;
- (void)setGenericFont:(NSFont*)font;
- (NSDictionary*)genericFontAttributes;
- (void)setGenericFontAttributes:(NSDictionary*)attrs;
@end

#pragma mark -
#pragma mark Document

extern NSString* const STStreamingFilesDidChangeNotification;

Protocol* STStreamingOutputInterfaceRef(void);
@protocol STStreamingOutputInterface

- (NSArray*)streamingOutputFileNames;
- (void)addUserStreamingOutputFileName:(NSString*)fileName;
- (void)removeUserStreamingOutputFileName:(NSString*)fileName;
- (void)setUserStreamingOutputFileName:(NSString*)fileName atIndex:(NSInteger)index;

@end

#pragma mark -
#pragma mark Inplace Text Editor Interface

@protocol STInPlaceTextEditDelegate;

@protocol STInPlaceTextEditableObjectInterface

- (void)startEditing;
- (void)endEditing;
- (NSString*)text;
- (void)setUserText:(NSString*)text;
- (NSDictionary*)textAttributes;
- (void)setUserTextAttributes:(NSDictionary*)textAttributes;
- (CGRect)textRectInLayer:(CALayer*)hostLayer;
- (CGPoint)textMargin;
- (NSTextAlignment)textAlignment;
- (void)setInPlaceEditDelegate:(id<STInPlaceTextEditDelegate>)delegate;
- (NSDictionary*)additionalSelectedTextAttributes;

@end

@protocol STInPlaceTextEditDelegate

- (void)startInPlaceEditingObject:(id<STInPlaceTextEditableObjectInterface>)object;
- (BOOL)isBeingEditedObject:(id)object;
- (void)stopInPlaceEditting:(id<STInPlaceTextEditableObjectInterface>)object shouldApplyChanges:(BOOL)shouldApplyChanges;

@end

#pragma mark -
#pragma mark Mouse Interfaces

@protocol STCursorHandlerInterface

- (NSCursor*)cursorForLocalPoint:(CGPoint)localPoint; // point in the object's coordinates
- (NSCursor*)cursorForPoint:(CGPoint)point; // point should be in super layer coordinates

@end

@protocol STMovingMouseResponderViewInterface

- (NSCursor*)cursorForPointInWindow:(NSPoint)point; // point should be in window coordinates
- (void)mouseMovedAtPointInWindow:(NSPoint)pointInWindow withModifierFlags:(NSUInteger)modifierFlags;

@end


@protocol STKeyDownEventResponderInterface

- (BOOL)acceptsKeyDown:(NSEvent*)keyDownEvent;

@end

@protocol STKeyDownEventMasterResponderInterface

- (void)keyDown:(NSEvent*)event;

@end

@protocol STKeyUpEventResponderInterface

- (BOOL)acceptsKeyUp:(NSEvent*)keyUpEvent;

@end

@protocol STKeyUpEventMasterResponderInterface

- (void)keyUp:(NSEvent*)event;

@end

@protocol STMouseDragViewDelegateInterface

- (void)draggableView:(NSView*)senderView didStartDraggingAtPoint:(NSPoint)pointInWindow;
- (void)draggableView:(NSView*)senderView isDraggingAtPoint:(NSPoint)pointInWindow startPoint:(NSPoint)startPoint;
- (void)draggableView:(NSView*)senderView didEndDraggingAtPoint:(NSPoint)pointInWindow startPoint:(NSPoint)startPoint;

@end

@protocol STMouseMovedPositionDelegateInterface

- (NSPoint)cursorPointInWindow;

@end

@protocol STTrackableObject

- (BOOL)startTrackingAtPoint:(CGPoint)point;
- (void)continueTrackingAtPoint:(CGPoint)point;
- (void)endTrackingAtPoint:(CGPoint)point;

@end

#pragma mark -
#pragma mark Main window layout

typedef NS_ENUM(NSInteger, STMainWindowLayout)
{
	STLayoutRightHanded = 0,
	STLayoutLeftHanded = 1,
	STLayoutMaxValue = STLayoutLeftHanded
};

extern NSInteger const STLayoutsPerKind;

@protocol STMainWindowLayoutInterface

- (STMainWindowLayout)mainWindowLayout;
- (void)setUserMainWindowLayout:(STMainWindowLayout)layout;

@end
