/*
 *  STProtocols.h
 *  Kahuna
 *
 *  Created by Davyd Geyl on 14/10/08.
 *  Copyright 2008 Sportstec Ltd. All rights reserved.
 *
 */

#import <SCNFoundation/STMacOSProtocols.h>

@class STColor;

#ifdef ST_NAME_REGISTER_ENABLED
@class STNameRegister;
@class STNameRegisterModel;
#endif

@protocol STProtoObjectInterface <NSObject>

@property (nonatomic, readonly) NSString* objectTitle;
@property (nonatomic, assign) id visualObject;

@end

Protocol* STDataContainerRef(void);

@protocol STDataContainer

@property (nonatomic, strong) id<STProtoObjectInterface> dataObject;

@end

typedef NS_ENUM(NSInteger, STCodingStatus)
{
	STCodingStatusCoding = 0,
	STCodingStatusDesign = 1 
};

typedef NS_ENUM(NSInteger, STTimelineExportFormat)
{
	STTimelineBinaryExportFormat = 0,
	STTimelineSportscodeXMLExportFormat = 1,
	STTimelineCSVExportFormat = 2
};

#pragma mark -
#pragma mark Holder

@protocol STHeldObjectInterface;

@protocol STHolderInterface <NSObject>

- (BOOL)canObjectBeRemoved:(id<STHeldObjectInterface>)object;
- (void)removeUserObject:(id<STHeldObjectInterface>)object;

@end

Protocol* STHeldObjectInterfaceRef(void);
@protocol STHeldObjectInterface

- (id<STHolderInterface>)holder;

@end

#pragma mark -
#pragma mark Framed Container

@protocol STFramedObjectInterface;
@protocol STFramedContainerInterface;

@protocol STNestedObjectInterface <NSObject>

@property (nonatomic, assign) id<STFramedContainerInterface> container;
- (BOOL)isNestedIn:(id<STFramedContainerInterface>)container;

@end


extern NSString* const STFramedContainerDidAddObjectsNotification;
extern NSString* const STFramedContainerDidRemoveObjectsNotification;
extern NSString* const STFramedContainerAffectedObjectsKey;

@protocol STFramedContainerInterface <STHolderInterface, STNestedObjectInterface>

@property (nonatomic, readonly) NSArray* objects;
@property (nonatomic, readonly) CGRect objectsFrame;

- (void)addObjects:(NSArray*)objects;
- (void)userAddObjects:(NSArray*)objects;
- (void)userAddObject:(id<STFramedObjectInterface>)object;

- (void)deleteObjects:(NSArray*)objects;
- (void)userDeleteObjects:(NSArray*)objects;
- (void)deleteObject:(id<STFramedObjectInterface>)object;
- (void)userDeleteObject:(id<STFramedObjectInterface>)object;

// Returns cumulative position. Object must be nested in the reciever. Container must be the receiver itself or a nesting container that owns the receiver somewhere down the tree.
//- (CGPoint)positionOfObject:(id<STFramedObjectInterface>)framedObject inContainer:(id<STFramedContainerInterface>)container;

- (CGPoint)positionInContainer:(id<STFramedContainerInterface>)container;

- (CGPoint)validatePosition:(CGPoint)position forObject:(id<STFramedObjectInterface>)framedObject;
- (CGRect)validateFrame:(CGRect)frame forObject:(id<STFramedObjectInterface>)framedObject;
- (void)objectDidChangeFrame:(id<STFramedObjectInterface>)framedObject;

@end

#pragma mark -
#pragma mark Framed Object

extern NSString* const STFramedObjectPositionDidChangeNotification;
extern NSString* const STFramedObjectPositionIsChangingNotification;
extern NSString* const STFramedObjectBoundsDidChangeNotification;
extern NSString* const STFramedObjectBoundsIsChangingNotification;
extern NSString* const STFramedObjectLockDidChangeNotification;
extern NSString* const STFramedObjectZPositionDidChangeNotification;

Protocol* STFramedObjectInterfaceRef(void);
@protocol STFramedObjectInterface <STProtoObjectInterface, STNestedObjectInterface>

- (CGRect)frame;
- (void)setUserFrame:(CGRect)frame;

// For continuos frame change. Notification is sent, but undo manager is not invoked.
- (void)setFrameResizing:(CGRect)frame;

// Finishes continuos frame change. Notification is sent and undo manager is invoked.
- (void)setUserFrame:(CGRect)frame previousFrame:(CGRect)previousFrame;

- (CGPoint)position;
- (void)setUserPosition:(CGPoint)position;

// For continuos position change. Notification is sent, but undo manager is not invoked.
- (void)setPositionChanging:(CGPoint)position;

// Finishes continuos position change. Notification is sent and undo manager is invoked.
- (void)setUserPosition:(CGPoint)position previousPosition:(CGPoint)previousPosition;

- (CGRect)bounds;
- (void)setBounds:(CGRect)bounds;
- (void)setUserBounds:(CGRect)bounds;

- (CGSize)minimalSize;
- (BOOL)allowChangeWidth;
- (BOOL)allowChangeHeight;

- (BOOL)isLocked;
- (void)setUserLocked:(BOOL)isLocked;

@property (nonatomic, assign) CGFloat zedOrder;

- (CGPoint)positionInContainer:(id<STFramedContainerInterface>)container;

@property (nonatomic, assign, getter=isUserLocked) BOOL userLocked;
@property (nonatomic, assign) CGFloat userPositionX;
@property (nonatomic, assign) CGFloat userPositionY;
@property (nonatomic, assign) CGFloat userFrameWidth;
@property (nonatomic, assign) CGFloat userFrameHeight;

@end

#pragma mark - Style Container

Protocol* STStyleContainerInterfaceRef(void);
@protocol STStyleContainerInterface

- (void)applyUserStyleWithObject:(id<STStyleContainerInterface>)object;

@end

// Movable Object
Protocol* STMovableObjectInterfaceRef(void);
@protocol STMovableObjectInterface

@property (nonatomic, assign) BOOL isBeingMoved;

- (void)startMovingAtPoint:(CGPoint)point;
- (void)movingAtPoint:(CGPoint)point;
- (void)endUserMovingAtPoint:(CGPoint)point;

@end

// Selectable Object

Protocol* STSelectableObjectInterfaceRef(void);
@protocol STSelectableObjectInterface

- (BOOL)isSelected;
- (void)setSelected:(BOOL)isSelected;

@end

// Highlightable Object

typedef NS_ENUM(NSInteger, STHighlightingType)
{
    STHighlightingDefault = 0,
    STHighlightingExclusiveGroup = 1,
	STHighlightingDuplication = 2
};

Protocol* STHighlightableObjectInterfaceRef(void);
@protocol STHighlightableObjectInterface

@property (nonatomic, getter = isHighlighted) BOOL highlighted;

@end

// Animated Object

extern NSString* const STAnimatedVisibleDidChangeNotification;
Protocol* STAnimatedVisibleInterfaceRef(void);

@protocol STAnimatedVisibleInterface

- (BOOL)isAnimatedVisible;
- (void)setAnimatedVisible:(BOOL)visible;

@end

// Durational object

extern NSString* const STContentDurationDidChangeNotification;

Protocol* STDurationalInterfaceRef(void);

@protocol STDurationalInterface

- (NSTimeInterval)contentDuration;
- (BOOL)isContentDurationUserEditable;
@property (nonatomic, assign) NSTimeInterval userContentDuration;

@end

extern NSString* const STGridVisibilityUserSettingKey;
extern NSString* const STGridVisibilityInterfaceDidChangeNotification;

Protocol* STGridVisibilityInterfaceRef(void);
@protocol STGridVisibilityInterface

@property (nonatomic, assign, getter=isGridHidden) BOOL gridHidden;
@property (nonatomic, assign, getter=isUserGridHidden) BOOL userGridHidden;

@end


#pragma mark -
#pragma mark Coding Object

@class STCodingModel;

Protocol* STCodingObjectInterfaceRef(void);
@protocol STCodingObjectInterface <NSObject>

@property (nonatomic, assign) STCodingModel* codingModel;

@end

Protocol* STVisualCodingObjectInterfaceRef(void);
@protocol STVisualCodingObjectInterface <NSObject>

- (void)performCodingActionAtPoint:(CGPoint)point;

@end

#pragma mark
#pragma mark Refresh Timer

extern NSString* const STRefreshTimerDidEndTimerNotification;
extern NSString* const STRefreshTimerDidStartTimerNotification;

Protocol* STRefreshTimerInterfaceRef(void);
@protocol STRefreshTimerInterface <NSObject>

@end

#pragma mark -
#pragma mark Code Name

extern NSString* const STCodeNameDidChangeNotification;

Protocol* STCodeNameInterfaceRef(void);
@protocol STCodeNameInterface

- (NSString*)codeName;
@property (nonatomic, strong) NSString *userCodeName;

- (NSString*)codeNameDescription;

- (STColor*)codeNameColor;
- (void)setUserCodeNameColor:(STColor*)color;

- (NSFont *)codeNameFont;
@property (nonatomic, strong) NSFont *userCodeNameFont;

@end

Protocol* STEventPropertiesInterfaceRef(void);

@protocol STEventPropertiesInterface
- (BOOL)isAutoPopulated;
@property (nonatomic, assign, getter = isUserAutoPopulated) BOOL userAutoPopulated;
@end

#pragma mark - Code Notes

typedef NS_ENUM(NSUInteger, STCodeNotesMode)
{
    STCodeNotesOnActivate = 0,
    STCodeNotesOnDeactivate = 1
};


extern NSString* const STCodeNotesDidChangeNotification;

Protocol* STCodeNotesInterfaceRef(void);
@protocol STCodeNotesInterface <NSObject>

@property (nonatomic, assign) BOOL codeNotesEnabled;
- (void)setUserCodeNotesEnabled:(BOOL)codeNotesEnabled;

@property (nonatomic, assign) STCodeNotesMode codeNotesMode;
- (void)setUserCodeNotesMode:(STCodeNotesMode)codeNotesMode;

@property (nonatomic, readonly) NSString* currentNotes;
- (void)setUserCurrentNotes:(NSString*)notes;

@property (nonatomic, readonly) NSArray* recentHistoryOfCodingNotes;
- (void) addRecentCodingNotes:(NSString*)string;

@end

#pragma mark -
#pragma mark Fill Interface

@class STImage;

typedef NS_ENUM(NSInteger, STFillType)
{
	STFillNone = 0,
	STFillColor = 1,
	STFillImage = 2,
	STFillGradient = 3
};

typedef NS_ENUM(NSUInteger, STImageFillType)
{
    STOriginal = 0,
    STScaleToFit = 1,
    STScaleToFill = 2,
    STStretch = 3,
    STTile = 4
};



extern NSString* const STFillObjectDidChangeNotification;
extern NSString* const STAvailableImageFillTypesDidChangeNotification;

extern CGFloat const STGradientAngleHorizontal;
extern CGFloat const STGradientAngleVertical;

Protocol* STFillObjectInterfaceRef(void);
@protocol STFillObjectInterface

- (STFillType)fillType;
- (NSArray*)availableFillTypes;
- (void)setUserFillType:(STFillType)fillType;

@property (nonatomic, strong) STColor *color;
@property (nonatomic, strong) STColor *userColor;
@property (nonatomic, strong) STColor *userNSColor;

@property (nonatomic, strong) STColor *gradientEndColor;
@property (nonatomic, strong) STColor *userGradientEndColor;
@property (nonatomic, strong) STColor *userGradientEndNSColor;


- (CGFloat)gradientAngle;
- (void)setUserGradientAngle:(CGFloat)angle;
- (STImage*)fillImage;
- (void)setUserFillImage:(STImage*)fillImage;
- (STImageFillType)imageFillType;
- (void)setUserImageFillType:(STImageFillType)imageFillType;
- (NSArray*)availableImageFillTypes;

@end

#pragma mark -
#pragma mark Bevel Interface

extern NSString* const STBevelDidChangeNotification;
extern NSString* const STBeveledObjectCodingKey;

Protocol* STBevelInterfaceRef(void);
@protocol STBevelInterface

@property (nonatomic, assign, getter = isBeveled) BOOL beveled;
@property (nonatomic, assign, getter = isUserBeveled) BOOL userBeveled;

@end

#pragma mark -
#pragma mark Stroke Interface

typedef NS_ENUM(NSInteger, STStrokePatternType)
{
    STStrokePatternSolid = 0,
    STStrokePatternDashed = 1,
    STStrokePatternDotted = 2,
    STStrokePatternDashedDotted = 3
};

extern NSString* const STStrokeObjectDidChangeNotification;

Protocol* STStrokeObjectInterfaceRef(void);
@protocol STStrokeObjectInterface

@property (nonatomic, strong) STColor *strokeColor;
@property (nonatomic, assign) STColor *userStrokeColor;
@property (nonatomic, assign) NSColor *userStrokeNSColor;

- (CGFloat)strokeWidth;
- (void)setStrokeWidth:(CGFloat)width;
@property (nonatomic, assign) CGFloat userStrokeWidth;

- (STStrokePatternType)strokePatternType;
- (void)setStrokePatternType:(STStrokePatternType)strokePatternType;
@property (nonatomic, assign) STStrokePatternType userStrokePatternType;

- (BOOL)canHideStroke;
- (BOOL)isStrokeHidden;
- (void)setStrokeHidden:(BOOL)hidden;
@property (nonatomic, assign, getter=isUserStrokeHidden) BOOL userStrokeHidden;

@end

#pragma mark -
#pragma mark Shapedobject Interface

typedef NS_ENUM(NSInteger, STObjectShape)
{
	STShapeRectangle = 0,
	STShapeRoundedRectangle = 1,
	STShapeEllipse = 2,
	STShapeOfImage = 3
};

extern NSString* const STShapeObjectDidChangeNotification;

Protocol* STShapeObjectInterfaceRef(void);
@protocol STShapeObjectInterface

- (STObjectShape)shape;
- (void)setUserShape:(STObjectShape)shape;

- (CGFloat)cornerRadius;
- (void)setUserCornerRadius:(CGFloat)cornerRadius;

- (BOOL)isShapeOfImageAvailable;

@end

// popupTags holder

extern NSString* const STPopupTagsHolderInterfaceDidChangeNotification;

Protocol* STPopupTagsHolderInterfaceRef(void);
@protocol STPopupTagsHolderInterface

- (NSArray*)popupTags;
- (NSArray*)popupTagLinks;
- (BOOL)arePopupTagsVisible;
- (void)setPopupTagsVisible:(BOOL)visible;
- (void)setUserPopupTagsVisible:(BOOL)visible;
- (NSInteger)popupTagMatWidth;
- (void)setUserPopupTagMatWidth:(NSInteger)width;
- (STColor*)popupTagMatColor;
- (void)setUserPopupTagMatColor:(STColor*)color;

@end

#pragma mark -
#pragma mark - Margin Holder

extern NSString* const STMarginDidChangeNotification; 

Protocol* STMarginHolderInterfaceRef(void);
@protocol STMarginHolderInterface 

@property (nonatomic, assign) CGFloat userMargin;
@property (nonatomic, assign) CGFloat margin;

@end

#pragma mark -
#pragma mark - STAutoCollapsingInterface

extern NSString* const STContainerDidChangeAutoCollapsingNotification;

Protocol* STAutoCollapsingInterfaceRef(void);
@protocol STAutoCollapsingInterface

@property (nonatomic, assign) BOOL userAutoCollapsing;

- (BOOL)autoCollapsing;

@end

#pragma mark -
#pragma mark - Show/Hide Knob

@protocol STKnobVisibleInCodingInterface

@property (nonatomic, assign) BOOL userKnobVisibleInCoding;

- (BOOL)isKnobVisibleInCoding;

@end

#pragma mark -
#pragma mark Timelines

extern NSString* const STTimelineInstanceStartTimeDidChangeNotification;
extern NSString* const STTimelineInstanceDurationDidChangeNotification;
extern NSString* const STTimelineInstanceDidChangeTagsNotification;
extern NSString* const STTimelineInstanceDidChangeNotesNotification;

Protocol* STTimelineInstanceInterfaceRef(void);
@protocol STTimelineInstanceInterface

- (NSTimeInterval)startTime;
- (void)setUserStartTime:(NSTimeInterval)time;
- (void)setStartTimeAndNotify:(NSTimeInterval)time;

- (NSTimeInterval)duration;
- (void)setUserDuration:(NSTimeInterval)duration;
- (void)setDurationAndNotify:(NSTimeInterval)duration;

- (BOOL)isEditable;

@property (nonatomic, strong) NSString* notes;
- (void)setUserNotes:(NSString*)notes;

@end

Protocol* STTimelineInstanceNudgeInterfaceRef(void);
@protocol STTimelineInstanceNudgeInterface <NSObject>

@property (nonatomic, assign, getter=isBeingNudged) BOOL beingNudged;

- (void)startNudgingAtTimeInterval:(NSTimeInterval)timeInterval;
- (void)nudgingAtTimeInterval:(NSTimeInterval)timeInterval;
- (void)endNudgingAtTimeInterval:(NSTimeInterval)timeInterval;

@end

#pragma mark -
#pragma mark Movie Attributes Interface

extern NSString* const STMovieDidChangeAttributesNotification;

Protocol* STMovieAttributesInterfaceRef(void);

@protocol STMovieAttributesInterface

- (NSString*)videoFormat;
- (NSString*)audioFormat;
- (CGFloat)FPS;
- (CGSize)originalSize;
- (long long)movieDataSizeBytes;
- (long long)dataSizeBytes;
- (NSTimeInterval)duration;
- (BOOL)isDurationEditable;
- (NSURL*)URL;

@end

#pragma mark -
#pragma mark Copy Paste Duplicate Interface

Protocol* STCopyPasteDuplicateIntefaceRef(void);

@protocol STCopyPasteDuplicateInteface

- (NSData*)dataWithCopyObjects:(NSArray*)objects;
- (void)pasteObjects:(NSArray*)objects;
- (void)pasteFromData:(NSData*)data;
- (void)duplicateObjects:(NSArray*)objectsToDuplicate;

@end

#ifdef ST_NAME_REGISTER_ENABLED

#pragma mark -
#pragma mark Name Register Interface

extern NSString* const STNameRegisterInterfaceDidChangeNotification;

Protocol* STNameRegisterInterfaceRef(void);
@protocol STNameRegisterInterface <NSObject>


@property (nonatomic, strong) STNameRegister* nameRegister;
@property (nonatomic, strong) STNameRegisterModel* nameRegisterModel;

@property (nonatomic, readonly) BOOL hasNameRegister;
@property (nonatomic, readonly) NSUInteger nameRegisterID;
@property (nonatomic, readonly) NSString* nameRegisterItemName;

- (void)setUserNameRegister:(STNameRegister*)nameRegister;

@end

#endif

