//
//  STTimelineChangeOperation.h
//  SCNFrameworks
//
//  Created by Yixiang Zhang on 27/4/17.
//  Copyright © 2017 Hudl. All rights reserved.
//

#import <Foundation/Foundation.h>

typedef NS_ENUM(NSUInteger, STTimelineOperationType)
{
    STTimelineOperationAddEvent = 101,
    STTimelineOperationDeleteEvent,
    STTimelineOperationAddInstance,
    STTimelineOperationAlterInstance,
    STTimelineOperationRemoveInstance,
    STTimelineOperationAddTag,
    STTimelineOperationRemoveTag,
};

@interface STTimelineChangeOperation : NSObject

// an ID to indicate the order of the operation on a local machine
@property (nonatomic, readonly) NSUInteger localID;

// the actual time of the operation in UTC, used to compare operation from different device
@property (nonatomic, readonly, copy, nonnull) NSString *timeStamp;

// Operation Type
@property (nonatomic, readonly) STTimelineOperationType operationType;

// The object's identifier of the operation.
// e.g. if to delete an event, this will be the identifier of the event to be deleted
@property (nonatomic, readonly, copy, nonnull) NSString *objectIdentifier;

// Provide more infomation of the operation
// e.g. if to add an instance, the real instance will be attached here
// Can be nil if this operation does not need.
@property (nonatomic, readonly, nullable) id userInfo;

@property (nonatomic, readonly, nullable) NSString *JSONStringRepresentation;

- (nullable instancetype)init NS_UNAVAILABLE;
- (nullable instancetype)initWithLocalID:(NSUInteger)localID
                               timeStamp:(NSString *_Nonnull)timeStamp
                           operationType:(STTimelineOperationType)operationType
                        objectIdentifier:(NSString *_Nonnull)objectIdentifer
                                userInfo:(id _Nullable)userInfo NS_DESIGNATED_INITIALIZER;
- (nullable instancetype)initWithJSON:(nonnull NSString *)JSONRepresentation;

@end
